﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                                Panorama                                  |
//|                              Version 1.00                                |
//|                                                                          |
//|            This project displays full 360° panorama pictures.            |
//|  Using either buttons or keyboard direction keys to change view angle.   |
//|                                                                          |
//|          The GDImage.dll unregistered version has no limitation,         |
//|             and all features of the API are fully functional.            |
//|                                                                          |
//|       Buying a commercial license removes the copyright nag screen.      |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|        You can find the latest version and all my C# demos there:        |
//|             http://www.zapsolution.com/winlift/ccorner.htm               |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2006                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 10-18-2006 (MM-DD-YYYY)            |
//|                        Last revised : 10-22-2006 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
//using System.Collections.Generic;
//using System.ComponentModel;
//using System.Data;
//using System.Drawing;
//using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;
//using System.Runtime;
//using System.IO;
//using Microsoft.Win32;

using Win32;
using GDImage;
using OpenGL;

namespace Panorama
{
    public partial class MAIN_Form : Form
    {

        const int ID_CTRL  = 100;                      // GDImage Zoom control
        const int TIMER_MAX = 6;                       // To perform smooth animation

        static public IntPtr hFORM_Main = IntPtr.Zero; // Default to Zero
        static public IntPtr hGLcontrol = IntPtr.Zero; // Default to Zero

        static GI.ZGLFONT UseFont = new GI.ZGLFONT();

        static public double Vert;
        static public double Horz;
        static public double vIncr;
        static public double hIncr;

        public MAIN_Form()
        {
            InitializeComponent();

            // Create GDImage control
            if (GI.RegisterGLImageClass())
            {
               
                // Create the OpenGL control
                hGLcontrol = GI.CreateControl(GL_Template, ID_CTRL);
                if (hGLcontrol != IntPtr.Zero)
                {
                    GI.DoNotSquareTexture(); // Do not fit the texture into a square shape
                    string Texture = Api.ExePath() + "pano.jpg";  // Use this fisheye panorama picture

                    if (GI.SetGLTextureFromFile(Texture) == 0)
                    {
                        GI.InitGLControl(Api.ARGB(255,255,255,255));
                        
                        ResetPano();

                        UseFont.fontName = "Arial";       // A True Type font
                        UseFont.fontHeight = 10;          // Size of the font
                        UseFont.fontWeight = Api.FW_BOLD; // Font style
                        GI.BuildGLfont(GI.GetGLDC(hGLcontrol), ref UseFont);

                        hFORM_Main = this.Handle;          // Save the FORM_Main handle

                        OnOff.Checked = true;

                        StartAnimation();                       
                    }
                }
            }
        }

        private void ResetPano()
        {
            Vert = 90;
            Horz = 180;
            vIncr = 0;
            hIncr = 0;
            Zoom_Slide.Value = 24;
        }

        private void StartAnimation()
        {
            // Use API WM_TIMER to avoid aggressive garbage collection 
            // to occur before Main ends.
            for (int T = 0; T < TIMER_MAX; T++)
            {
                Api.SetTimer(hFORM_Main, T + 1, 0, null);
            }
        }

        protected override void WndProc(ref Message m)
        {
            switch (m.Msg)
            {               
                case Api.WM_TIMER:
                    RenderOpenGL(hGLcontrol);
                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_DESTROY:
                    DisposeGL(); // Free up OpenGL resources
                    break;
            }
            base.WndProc(ref m);

            if (m.Msg == Api.WM_COMMAND)
            {
                // Don't want the track bar to gain focus over my OpenGL control
                Api.SetFocus(m.HWnd);
            }
        }

        // Free up resources
        private void DisposeGL()
        {
            // Killtimer(s)
            for (int T = 0; T < TIMER_MAX; T++)
            {
                Api.KillTimer(hFORM_Main, T + 1);
            }
            // Delete OpenGL font.
            GI.DeleteGLFont(ref UseFont);
        }

        private void RenderOpenGL(IntPtr hContainer)
        {
            GL.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);

            int V, H;

            Vert += vIncr;
            Horz += hIncr;
            if (OnOff.Checked)
            {
                V = (int) Vert % 360;
                H = (int) Horz % 360;
                
                int UseColor = Api.ARGB(255, 128, 0, 0);

                string MyStr = "Horizontal " + Math.Abs(H).ToString();
                GI.DrawGLText(hContainer, ref UseFont, 10, 10, MyStr, UseColor);

                MyStr = "Vertical " + V.ToString();
                GI.DrawGLText(hContainer, ref UseFont, 26, 10 + UseFont.fontHeight, MyStr, UseColor);

                MyStr = "Zoom " + (101 - Math.Truncate(GI.ZI_GetGLzoom(hContainer) * 1.72415)).ToString();
                GI.DrawGLText(hContainer, ref UseFont, 40, 10 + UseFont.fontHeight * 2, MyStr, UseColor);
            }

            GL.glPushMatrix();

            //*******************************
            GL.UsePolarView(1, 0, Vert, Horz);
            int quadObj = GL.gluNewQuadric();                  // Pointer to the Quadric Object (Return 0 If No Memory))
            if (quadObj != 0)
            {
                GL.gluQuadricNormals(quadObj, GL.GLU_SMOOTH);  // Create Smooth Normals
                GL.gluQuadricTexture(quadObj, GL.GL_TRUE);     // Create Texture Coords
                GL.gluSphere(quadObj, 2, 48, 48);
                GL.gluDeleteQuadric(quadObj);
            }
            //*******************************

            GL.glPopMatrix();

            GI.UpdateGLWindow(hContainer);
        }

        // ****************************************
        private void FORM_Main_Close(object sender, EventArgs e)
        {
        }

        //***********************************************************
        // We use the .NET template to resize the GDImage control
        private void FORM_Main_Resize(object sender, EventArgs e)
        {
            // Use the template to resize the GDImage OpenGL control
            Api.MoveWindow(hGLcontrol, GL_Template.Location.X, GL_Template.Location.Y, GL_Template.Width, GL_Template.Height, true);
            GI.ResizeGLWindow(hGLcontrol);
            RenderOpenGL(hGLcontrol);
        }

        private void BTN_Reset_Click(object sender, EventArgs e)
        {
            ResetPano();
            GI.SetGLzoom(hGLcontrol, 0);
            GI.ResizeGLWindow(hGLcontrol);
            RenderOpenGL(hGLcontrol);
        }

        private void Zoom_Slide_Scroll(object sender, EventArgs e)
        {
            int SetZoom = (int) Math.Truncate(Zoom_Slide.Value / 1.72415);
            GI.SetGLzoom(hGLcontrol, 58 - SetZoom);
            GI.ResizeGLWindow(hGLcontrol);
        }

        private void BTN_Start_Click(object sender, EventArgs e)
        {
            hIncr = 0.05;
        }

        private void BTN_Up_Click(object sender, EventArgs e)
        {
            vIncr += 0.05;
        }

        private void BTN_Left_Click(object sender, EventArgs e)
        {
            hIncr -= 0.05;
        }

        private void BTN_Right_Click(object sender, EventArgs e)
        {
            hIncr += 0.05;
        }

        private void BTN_Down_Click(object sender, EventArgs e)
        {
            vIncr -= 0.05;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            vIncr = 0.0; hIncr = 0.0;
        }

        private void Form_KeyDown(object sender, KeyEventArgs e)
        {
            bool Eat = false;
            switch ((int)e.KeyCode)
            {
                case Api.VK_LEFT:
                case Api.VK_RIGHT:
                case Api.VK_UP:
                case Api.VK_DOWN:
                case Api.VK_PGUP:
                case Api.VK_PGDN:
                    Eat = true;
                    break;
                default: break;
            }
            e.Handled = Eat;
        }

        private void Form_PreviewKeydown(object sender, PreviewKeyDownEventArgs e)
        {
            int ZoomIs;
            switch ((int)e.KeyCode)
            {
                case Api.VK_LEFT: hIncr += 0.01; break;
                case Api.VK_RIGHT: hIncr -= 0.01; break;
                case Api.VK_UP: vIncr += 0.01; break;
                case Api.VK_DOWN: vIncr -= 0.01; break;
                case Api.VK_PGUP:
                    ZoomIs = GI.GetGLzoom(hGLcontrol);
                    if (ZoomIs > 1)
                    {
                        GI.SetGLzoom(hGLcontrol, ZoomIs - 1);
                        GI.ResizeGLWindow(hGLcontrol); 
                    }
                    break;
                case Api.VK_PGDN:
                    ZoomIs = GI.GetGLzoom(hGLcontrol);
                    if (ZoomIs < 58)
                    {
                        GI.SetGLzoom(hGLcontrol, ZoomIs + 1);
                        GI.ResizeGLWindow(hGLcontrol); 
                    }
                    break;
            }
        }

        private void BTN_Select_Click(object sender, EventArgs e)
        {
            GI.LoadSavePath(Api.ExePath(), true);
            string Texture = GI.LoadDialog(hFORM_Main);
            if (Texture != null)
            {
	            GI.DoNotSquareTexture();          // Do not fit the texture into a square shape.
                GI.SetGLTextureFromFile(Texture); // Use this new texture.
            }

        }
    }
}